using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TeamSystem.Data.DataGenerator;
using System.IO;
using System.ComponentModel;
using System.Windows.Forms.Design;
using System.Drawing.Design;

namespace CustomDataGenerators
{
    public class FileSourceGenerator: Generator
    {
        private string _path;           //Holds the path to the file with the data
        private int _currentRecord;     //Pointer to the current record
        private List<string> _values;   //Stores all of the values from the file

        /// <summary>
        /// Allows the user to set the file path
        /// </summary>
        [Input(Name = "File Path", Description = "The path to the file containing the values to use in data generation.", EditorType = typeof(FileNameEditor), TypeConverter = typeof(StringConverter))]
        public string FilePath
        {
            get
            {
                return _path;
            }
            set
            {
                _path = value;
            }
        }
      
        /// <summary>
        /// This method is called each time data needs to be generated (i.e. like
        /// when the user selects Data Generation Preview tab)
        /// </summary>
        /// <param name="initInfo">Provides some basic information on the generation</param>
        protected override void OnInitialize(GeneratorInit initInfo)
        {
            _values = new List<string>();

            FileStream fs = new FileStream(_path, FileMode.Open, FileAccess.Read);
            StreamReader sr = new StreamReader(fs);

            while (!sr.EndOfStream)
            {
                _values.Add(sr.ReadLine());
            }

            sr.Close();
        }

        /// <summary>
        /// Returns the value that is generated. "Result" will appear in the Generator
        /// Output column of the Column Details window
        /// </summary>
        [Output(Name="Result")]
        public string Result
        {
            get
            {
                if (_currentRecord >= _values.Count)
                    _currentRecord = 0;

                return _values[_currentRecord];
            }
        }

        /// <summary>
        /// Generates the next value - in this case we just increment the pointer to 
        /// a value in our list.
        /// </summary>
        protected override void OnGenerateNextValues()
        {
            _currentRecord++;
        }

        /// <summary>
        /// Called before data is generated. Here we are only checking to see that the file exists.
        /// This exception will show up in the Error List when you select the Data
        /// Generation preview tab.
        /// </summary>
        protected override void OnValidateInputs()
        {
            if (!File.Exists(_path))
                throw new InputValidationException("The specified file does not exist.");
        }
    }
}