using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TeamSystem.Data.DataGenerator;
using System.Data;
using System.Data.SqlClient;
using System.ComponentModel;

namespace CustomDataGenerators
{
    [GeneratorStyles(DesignerStyles = GeneratorDesignerStyles.None),
     Generator(typeof(SequentialGeneratorDesigner))]
    public class SequentialDataBoundGenerator : Generator
    {
        #region Private Members
        private string _connectionInfo = "";
        private string _selectStatement = "";
        private DataSet _ds;
        private int _currentRecord = 0;
        #endregion

        #region Inputs
        [Input(Name = "Select Statement")]
        public string SelectStatement
        {
            get { return _selectStatement; }
            set { _selectStatement = value; }
        }

        [Input(EditorType=typeof(System.Web.UI.Design.ConnectionStringEditor),TypeConverter=typeof(StringConverter),Name = "Connection Info")]
        public string ConnectionInfo
        {
            get { return _connectionInfo; }
            set { _connectionInfo = value; }
        }
        #endregion

        protected override void OnGenerateNextValues()
        {
            _currentRecord++;
            if (_currentRecord >= _ds.Tables[0].Rows.Count)
                _currentRecord = 0;
        }

        protected override object OnGetOutputValue()
        {
            return  _ds.Tables[0].Rows[_currentRecord][StripBrackets(base.OutputKey)];
        }

        protected override void OnInitialize(GeneratorInit initInfo)
        {
            SqlConnection cn = new SqlConnection(_connectionInfo);
            SqlCommand cmd = new SqlCommand(_selectStatement, cn);
            SqlDataAdapter da = new SqlDataAdapter(cmd);
            _ds = new DataSet();
            da.Fill(_ds);
        }

        protected override void OnValidateInputs()
        {
            if (_connectionInfo.Length == 0)
                throw new InputValidationException("Connection Info must be supplied.");
            if (_selectStatement.Length == 0)
                throw new InputValidationException("Select statement must be supplied.");
        }

        private string StripBrackets(string column)
        {
            return column.Substring(1, column.Length - 2);
        }
    }
}