using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TeamSystem.Data.DataGenerator;
using System.Data;
using System.Data.SqlClient;

namespace CustomDataGenerators
{
    public class SequentialGeneratorDesigner : DefaultGeneratorDesigner
    {
        private string _cn;                 //Holds the connection string
        private string _select;             //Holds the select statement
        private OutputDescriptor[] _cache;  //Cache's the output

        /// <summary>
        /// Fired when one of the properties decorated an InputAttribute is changed
        /// </summary>
        public override void OnInputValueChanged(object sender, InputChangedEventArgs eventArgs)
        {
            if (eventArgs.Input.Key == "ConnectionInfo")
            {
                _cn = eventArgs.Input.Value as string;
            }

            if (eventArgs.Input.Key == "SelectStatement")
            {
                _select = eventArgs.Input.Value as string;
            }
        }

        /// <summary>
        /// Called whenever the Output column of the Column Details window needs to 
        /// be populated
        /// </summary>
        /// <returns>The list of columns from the select statement.</returns>
        public override OutputDescriptor[] GetOutputs()
        {
            if (_cn != null && _select != null)
            {
                DataTable dt = new DataTable();
                SqlConnection cn = new SqlConnection(_cn);
                SqlCommand cmd = new SqlCommand(_select, cn);
                SqlDataReader dr;

                cn.Open();
                cmd.CommandType = CommandType.Text;
                dr = cmd.ExecuteReader(CommandBehavior.SchemaOnly);
                dt = dr.GetSchemaTable();
                cn.Close();

                List<OutputDescriptor> od = new List<OutputDescriptor>();
                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    if (dt.Rows[i].Table.Columns.Contains("ColumnName"))
                    {
                        OutputDescriptor desc = new OutputDescriptor("[" + dt.Rows[i]["ColumnName"].ToString() + "]", (Type)dt.Rows[i]["DataType"]);
                        od.Add(desc);
                    }
                }
                _cache = od.ToArray();
            }
            return _cache;
        }
    }
}